//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class	        G3DSphere
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DSphere.h,v 1.9 2001/01/05 20:50:48 robert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DSphere_h_INCLUDE
#define __G3DSphere_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DIntersecting.h>

@class G3DMatrix3f;
@class G3DMatrix4f;
@class G3DTuple3f;
@class G3DBox;
@class G3DLine;
@class G3DPlane;

@interface G3DSphere : NSObject <NSCoding, NSCopying, G3DIntersecting>
{
  @public
    
  float radius;
  float center[3];
}

//-----------------------------------------------------------------------------
// class methods
//-----------------------------------------------------------------------------

+ (G3DSphere *)sphere;
+ (G3DSphere *)sphereWithCenter:(G3DTuple3f *)aCenter radius:(float)aRadius;
+ (G3DSphere *)sphereWithX:(float)x y:(float)y z:(float)z radius:(float)aRadius;
+ (G3DSphere *)sphereWithSphere:(G3DSphere *)aSphere;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithCenter:(G3DTuple3f *)aCenter radius:(float)aRadius;
- (id)initWithX:(float)x y:(float)y z:(float)z radius:(float)aRadius;
- (id)initWithSphere:(G3DSphere *)aSphere;

//-----------------------------------------------------------------------------
// math
//-----------------------------------------------------------------------------

- (void)wrapPoint:(G3DTuple3f *)pt;
- (void)wrapBox:(G3DBox *)b;
- (void)wrapSphere:(G3DSphere *)s;

- (void)transform:(G3DMatrix4f *)m;
  // Calculates m * center, such as it can be used by OpenGL. The radius is scaled by the biggest scaling factor.

- (BOOL)intersectsObject:(id)anObject;

- (BOOL)intersectsLine:(G3DLine *)l;
- (BOOL)intersectsSphere:(G3DSphere *)s;
- (BOOL)intersectsBox:(G3DBox *)b;
- (BOOL)intersectsPlane:(G3DPlane *)p;

//-----------------------------------------------------------------------------
// accessor methods
//-----------------------------------------------------------------------------

- (float)volume;
- (BOOL)isEmpty;

- (float)radius;
- (void)setRadius:(float)newRadius;

- (G3DTuple3f *)center;
- (void)setCenter:(G3DTuple3f *)newCenter;

- (BOOL)isEqualToSphere:(G3DSphere *)aSphere;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

@end

extern NSString *G3DSphereException;

#endif




